@@ -1,7 +1,7 @@
 /*
     SPDX-FileCopyrightText: 2017 David Edmundson <davidedmundson@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "runnermanager.h"
@@ -37,6 +37,8 @@ public:
     DBusRunnerTest();
     ~DBusRunnerTest();
 
+    const QString dbusRunnerBinary = QStringLiteral(DBUS_RUNNER_BINARY);
+
 private Q_SLOTS:
     void initTestCase();
 #if WITH_KSERVICE
@@ -44,7 +46,10 @@ private Q_SLOTS:
 #endif
     void testMatch();
     void testMulti();
+    void testFilterProperties();
+    void testFilterProperties_data();
     void testRequestActionsOnce();
+    void testDBusRunnerSyntaxIntegration();
 #if WITH_KSERVICE
     void testMatch_data();
     void testMulti_data();
@@ -66,6 +71,7 @@ DBusRunnerTest::~DBusRunnerTest()
 
 void DBusRunnerTest::initTestCase()
 {
+    QVERIFY(QFileInfo::exists(dbusRunnerBinary));
     // Set up a layer in the bin dir so ksycoca & KPluginMetaData find the Plasma/Runner service type
     const QByteArray defaultDataDirs = qEnvironmentVariableIsSet("XDG_DATA_DIRS") ? qgetenv("XDG_DATA_DIRS") : QByteArray("/usr/local:/usr");
     const QByteArray modifiedDataDirs = QFile::encodeName(QCoreApplication::applicationDirPath()) + QByteArrayLiteral("/data:") + defaultDataDirs;
@@ -115,7 +121,7 @@ void DBusRunnerTest::testMatch()
 #endif
 
     QProcess process;
-    process.start(QFINDTESTDATA("testremoterunner"), QStringList({QStringLiteral("net.krunnertests.dave")}));
+    process.start(dbusRunnerBinary, QStringList({QStringLiteral("net.krunnertests.dave")}));
     QVERIFY(process.waitForStarted());
 
     QTest::qSleep(500);
@@ -161,12 +167,12 @@ void DBusRunnerTest::testMatch()
     QSignalSpy processSpy(&process, &QProcess::readyRead);
     m.run(result);
     processSpy.wait();
-    QCOMPARE(process.readAllStandardOutput().trimmed(), QByteArray("Running:id1:"));
+    QCOMPARE(process.readAllStandardOutput().trimmed().split('\n').constLast(), QByteArray("Running:id1:"));
 
     result.setSelectedAction(action);
     m.run(result);
     processSpy.wait();
-    QCOMPARE(process.readAllStandardOutput().trimmed(), QByteArray("Running:id1:action1"));
+    QCOMPARE(process.readAllStandardOutput().trimmed().split('\n').constLast(), QByteArray("Running:id1:action1"));
 
     process.kill();
     process.waitForFinished();
@@ -189,11 +195,11 @@ void DBusRunnerTest::testMulti()
 #endif
 
     QProcess process1;
-    process1.start(QFINDTESTDATA("testremoterunner"), QStringList({QStringLiteral("net.krunnertests.multi.a1")}));
+    process1.start(dbusRunnerBinary, QStringList({QStringLiteral("net.krunnertests.multi.a1")}));
     QVERIFY(process1.waitForStarted());
 
     QProcess process2;
-    process2.start(QFINDTESTDATA("testremoterunner"), QStringList({QStringLiteral("net.krunnertests.multi.a2")}));
+    process2.start(dbusRunnerBinary, QStringList({QStringLiteral("net.krunnertests.multi.a2")}));
     QVERIFY(process2.waitForStarted());
 
     QTest::qSleep(500);
@@ -220,8 +226,8 @@ void DBusRunnerTest::testMulti()
     //verify matches, must be one from each
     QCOMPARE(m.matches().count(), 2);
 
-    QString first = m.matches().at(0).data().toString();
-    QString second = m.matches().at(1).data().toString();
+    const QString first = m.matches().at(0).data().toList().constFirst().toString();
+    const QString second = m.matches().at(1).data().toList().constFirst().toString();
     QVERIFY(first != second);
     QVERIFY(first == QLatin1String("net.krunnertests.multi.a1") || first == QStringLiteral("net.krunnertests.multi.a2"));
     QVERIFY(second == QLatin1String("net.krunnertests.multi.a1") || second == QStringLiteral("net.krunnertests.multi.a2"));
@@ -249,7 +255,7 @@ void DBusRunnerTest::testRequestActionsOnce()
 #endif
 
     QProcess process;
-    process.start(QFINDTESTDATA("testremoterunner"), QStringList({QStringLiteral("net.krunnertests.dave")}));
+    process.start(dbusRunnerBinary, QStringList({QStringLiteral("net.krunnertests.dave")}));
     QVERIFY(process.waitForStarted());
     QTest::qSleep(500);
 
@@ -278,17 +284,74 @@ void DBusRunnerTest::testRequestActionsOnce()
     // Construct a fake match with necesarry data
     QueryMatch fakeMatch(m.runner(QStringLiteral("dbusrunnertest")));
     fakeMatch.setId(QStringLiteral("dbusrunnertest_id1"));
-    fakeMatch.setData(QStringLiteral("net.krunnertests.dave"));
+    fakeMatch.setData(QVariantList({
+                                       QStringLiteral("net.krunnertests.dave"),
+                                       QStringList({QStringLiteral("action1"), QStringLiteral("action2")})
+                                   }));
 
     // We haven't called the prepare slot, if the implementation works
     // the actions should alredy be available
     auto actions = m.actionsForMatch(fakeMatch);
-    QCOMPARE(actions.count(), 1);
+    QCOMPARE(actions.count(), 2);
 
     process.kill();
     process.waitForFinished();
 }
 
+void DBusRunnerTest::testFilterProperties_data()
+{
+    QTest::addColumn<QString>("rejectedQuery");
+    QTest::addColumn<QString>("acceptedQuery");
+
+    QTest::newRow("min-letter-count") << "fo" << "foo";
+    QTest::newRow("match-regex") << "barfoo" << "foobar";
+}
+
+void DBusRunnerTest::testFilterProperties()
+{
+    QFETCH(QString, rejectedQuery);
+    QFETCH(QString, acceptedQuery);
+    QProcess process;
+    process.start(dbusRunnerBinary, QStringList({QStringLiteral("net.krunnertests.dave")}));
+    QVERIFY(process.waitForStarted());
+
+    QTest::qSleep(500);
+
+    RunnerManager m;
+    auto md = KPluginMetaData::fromDesktopFile(QFINDTESTDATA("dbusrunnertest.desktop"), {QStringLiteral("plasma-runner.desktop")});
+    QVERIFY(md.isValid());
+    m.loadRunner(md);
+    m.launchQuery(rejectedQuery);
+    QSignalSpy spy(&m, &RunnerManager::matchesChanged);
+    QVERIFY(spy.wait());
+    // Match method was not called, because the min letter count is 3
+    QVERIFY(process.readAllStandardOutput().isEmpty());
+
+    m.launchQuery(acceptedQuery);
+    QSignalSpy spy2(&m, &RunnerManager::matchesChanged);
+    QVERIFY(spy2.wait());
+    QCOMPARE(process.readAllStandardOutput().trimmed(), QString(QStringLiteral("Matching:") + acceptedQuery).toLocal8Bit());
+    process.kill();
+    process.waitForFinished();
+}
+
+void DBusRunnerTest::testDBusRunnerSyntaxIntegration()
+{
+    RunnerManager m;
+    KPluginMetaData md = KPluginMetaData::fromDesktopFile(QFINDTESTDATA("dbusrunnertest.desktop"), {QStringLiteral("plasma-runner.desktop")});
+    QVERIFY(md.isValid());
+    m.loadRunner(md);
+    const QList<RunnerSyntax> syntaxes = m.runner(md.pluginId())->syntaxes();
+    QCOMPARE(syntaxes.size(), 2);
+
+    QCOMPARE(syntaxes.at(0).exampleQueries().size(), 1);
+    QCOMPARE(syntaxes.at(0).exampleQueries().constFirst(), QStringLiteral("syntax1"));
+    QCOMPARE(syntaxes.at(0).description(), QStringLiteral("description1"));
+    QCOMPARE(syntaxes.at(1).exampleQueries().size(), 1);
+    QCOMPARE(syntaxes.at(1).exampleQueries().constFirst(), QStringLiteral("syntax2"));
+    QCOMPARE(syntaxes.at(1).description(), QStringLiteral("description2"));
+}
+
 QTEST_MAIN(DBusRunnerTest)
 
 
