@@ -1,42 +1,32 @@
 /*
- *   Copyright 2006-2007 Aaron Seigo <aseigo@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU Library General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #ifndef PLASMA_ABSTRACTRUNNER_H
 #define PLASMA_ABSTRACTRUNNER_H
 
 #include <QObject>
-#include <QMutex>
 #include <QStringList>
 #include <QIcon>
 
-#include <kconfiggroup.h>
-#include <kservice.h>
-#include <kplugininfo.h>
+#include <KConfigGroup>
+#include <KService>
+#include <KPluginInfo>
+#include <KPluginMetaData>
+
+#include <plasma_version.h>
+#include <plasma/plasma_export.h> // for PLASMA_ENABLE_DEPRECATED_SINCE
 
 #include "krunner_export.h"
 #include "querymatch.h"
 #include "runnercontext.h"
 #include "runnersyntax.h"
-#include <plasma_version.h>
 
 class QAction;
 class QMimeData;
+class QRegularExpression;
 
 namespace Plasma
 {
@@ -46,6 +36,8 @@ class Package;
 class QueryMatch;
 class AbstractRunnerPrivate;
 
+enum RunnerReturnPluginMetaDataConstant { RunnerReturnPluginMetaData }; // KF6: remove again
+
 /**
  * @class AbstractRunner abstractrunner.h <KRunner/AbstractRunner>
  *
@@ -67,11 +59,14 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
     Q_PROPERTY(QString description READ description)
     Q_PROPERTY(QString name READ name)
     Q_PROPERTY(QIcon icon READ icon)
+    Q_PROPERTY(int minLetterCount READ minLetterCount WRITE setMinLetterCount)
+    Q_PROPERTY(QRegularExpression matchRegex READ matchRegex WRITE setMatchRegex)
+
     public:
         /** Specifies a nominal speed for the runner */
         enum Speed {
             SlowSpeed,
-            NormalSpeed
+            NormalSpeed,
         };
 
         /** Specifies a priority for the runner */
@@ -80,7 +75,7 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
             LowPriority,
             NormalPriority,
             HighPriority,
-            HighestPriority
+            HighestPriority,
         };
 
         /** An ordered list of runners */
@@ -110,12 +105,11 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          * to have a separate class doing all the work like so:
          *
          * \code
-         * void MyFancyAsyncRunner::match( RunnerContext& context )
+         * void MyFancyAsyncRunner::match(RunnerContext &context)
          * {
          *     QEventLoop loop;
-         *     MyAsyncWorker worker( context );
-         *     connect( &worker, SIGNAL(finished()),
-         *              &loop, SLOT(quit()) );
+         *     MyAsyncWorker worker(context);
+         *     connect(&worker, &MyAsyncWorker::finished, &loop, &MyAsyncWorker::quit);
          *     worker.work();
          *     loop.exec();
          * }
@@ -147,21 +141,29 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         void performMatch(Plasma::RunnerContext &context);
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 71)
         /**
          * If the runner has options that the user can interact with to modify
          * what happens when run or one of the actions created in match
          * is called, the runner should return true
+         * @deprecated Since 5.0, this feature has been defunct
          */
+        KRUNNER_DEPRECATED_VERSION_BELATED(5, 71,  5, 0, "No longer use, feature removed")
         bool hasRunOptions();
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 71)
         /**
          * If hasRunOptions() returns true, this method may be called to get
          * a widget displaying the options the user can interact with to modify
          * the behaviour of what happens when a given match is selected.
          *
          * @param widget the parent of the options widgets.
+         * @deprecated Since 5.0, this feature has been defunct
          */
+        KRUNNER_DEPRECATED_VERSION_BELATED(5, 71,  5, 0, "No longer use, feature removed")
         virtual void createRunOptions(QWidget *widget);
+#endif
 
         /**
          * Called whenever an exact or possible match associated with this
@@ -173,6 +175,7 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         virtual void run(const Plasma::RunnerContext &context, const Plasma::QueryMatch &match);
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
         /**
          * Return a list of categories that this runner provides. By default
          * this list just contains the runners name. It is used by the runner manager
@@ -181,17 +184,24 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          *
          * This list of categories is also used to provide a better way to
          * configure the runner instead of the typical on / off switch.
+         * @deprecated Since 5.76, feature is unused. You can still set the category property in the QueryMatch
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "Feature is unused")
         virtual QStringList categories() const;
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
         /**
          * Returns the icon which accurately describes the category \p category.
          * This is meant to be used in a configuration dialog when showing
          * all the categories.
          *
          * By default this returns the icon of the AbstractRunner
+         * * @deprecated Since 5.0, feature removed
          */
+        KRUNNER_DEPRECATED_VERSION_BELATED(5, 76,  5, 0, "No longer use, feature removed")
         virtual QIcon categoryIcon(const QString& category) const;
+#endif
 
         /**
          * The nominal speed of the runner.
@@ -205,18 +215,40 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         Priority priority() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
          * Returns the OR'ed value of all the Information types (as defined in RunnerContext::Type)
          * this runner is not interested in.
          * @return OR'ed value of black listed types
+         * @deprecated This feature is deprecated
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "feature is deprecated")
         RunnerContext::Types ignoredTypes() const;
+#endif
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
-         * Sets the types this runner will ignore
+         * Sets the types this runner will ignore. If the value from RunnerContext::type() is contained in the ignored types
+         * the match() method won't be called. This way there is no unnecessary thread spawned. The same RunnerContext from
+         * which the type gets read is later passed into the match(Plasma::RunnerContext &context) method call.
          * @param types OR'ed listed of ignored types
+         * @deprecated feature is deprecated. Consider using the minLetterCount and matchRegex properties instead. These
+         * properties also prevent thread spawning, but can be used far more precise.
+         * If you want to have this kind of optimization for older KRunner versions you could wrap this
+         * inside of an version if statement:
+         * @code
+             #if KRUNNER_VERSION < QT_VERSION_CHECK(5, 76, 0)
+             //set ignore types
+             #endif
+         * @endcode
+         * The minLetterCount and matchRegex can be set with a similar version check or added to the desktop file.
+         * If an older KRunner version loads such a desktop file these unknown properties are just ignored.
+         * @see minLetterCount
+         * @see matchRegex
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "feature is deprecated. Consider using the minLetterCount and matchRegex properties instead")
         void setIgnoredTypes(RunnerContext::Types types);
+#endif
 
         /**
           * @return the user visible engine name for the Runner
@@ -233,16 +265,42 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
           */
         QString description() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
         /**
          * @return the plugin info for this runner
+         * @deprecated since 5.72, use metaData(Plasma::RunnerReturnPluginMetaDataConstant) instead, see its API docs
          */
+        KRUNNER_DEPRECATED_VERSION(5, 72, "Use metaData(Plasma::RunnerReturnPluginMetaDataConstant) instead, see its API docs")
         KPluginInfo metadata() const;
+#endif
 
+        /**
+         * @return the plugin metadata for this runner
+         *
+         * Overload to get non-deprecated metadata format. Use like this:
+         * @code
+         * KPluginMetaData md = runner->metadata(Plasma::RunnerReturnPluginMetaData);
+         * @endcode
+         * If you disable the deprecated version using the KRUNNER_DISABLE_DEPRECATED_BEFORE_AND_AT macro,
+         * then you can omit Plasma::RunnerReturnPluginMetaDataConstant and use it like this:
+         * @code
+         * KPluginMetaData md = runner->metadata();
+         * @endcode
+         *
+         * @since 5.72
+         */
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
+        KPluginMetaData metadata(RunnerReturnPluginMetaDataConstant) const;
+#else
+        KPluginMetaData metadata(RunnerReturnPluginMetaDataConstant = RunnerReturnPluginMetaData) const;
+#endif
         /**
          * @return the icon for this Runner
          */
         QIcon icon() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 65) // not 5.28 here, this KRUNNER visibility control only added at 5.65
+#if PLASMA_ENABLE_DEPRECATED_SINCE(5, 28) // Plasma::Package is defined with this condition
         /**
          * Accessor for the associated Package object if any.
          *
@@ -250,8 +308,12 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          * the runner.
          *
          * @return the Package object, which may be invalid
+         * @deprecated since 5.28, use KPackage::Package instead, no accessor in this class
          **/
+        KRUNNER_DEPRECATED_VERSION(5, 28, "No longer use, feature removed")
         Package package() const;
+#endif
+#endif
 
         /**
          * Signal runner to reload its configuration.
@@ -264,12 +326,16 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         QList<RunnerSyntax> syntaxes() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
          * @return the default syntax for the runner or @c nullptr if no default syntax has been defined
          *
          * @since 4.4
+         * @deprecated Since 5.76, feature is unused.
          */
+        KRUNNER_DEPRECATED_VERSION(5, 76, "No longer use, feature is unused")
         RunnerSyntax *defaultSyntax() const;
+#endif
 
         /**
          * @return true if the runner is currently busy with non-interuptable work, signaling that
@@ -278,6 +344,60 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         bool isMatchingSuspended() const;
 
+        /**
+         * This is the minimum letter count for the query. If the query is shorter than this value
+         * and KRunner is not in the singleRunnerMode, the performMatch and consequently match method is not called.
+         * This can be set using the X-Plasma-Runner-Min-Letter-Count property or the setMinLetterCount method.
+         * @see setMinLetterCount
+         * @see match
+         * @see performMatch
+         * @return minLetterCount property
+         * @since 5.75
+         */
+        int minLetterCount() const;
+
+        /**
+         * Set the minLetterCount property
+         * @param count
+         * @since 5.75
+         */
+        void setMinLetterCount(int count);
+
+        /**
+         * If this regex is set with a not empty pattern it must match the query in
+         * order for the performMatch/match being called.
+         * Just like the minLetterCount property this check is ignored when the runner is in the singleRunnerMode.
+         * In case both the regex and the letter count is set the letter count is checked first.
+         * @return matchRegex property
+         * @see hasMatchRegex
+         * @since 5.75
+         */
+        QRegularExpression matchRegex() const;
+
+        /**
+         * Set the matchRegex property
+         * @param regex
+         * @since 5.75
+         */
+        void setMatchRegex(const QRegularExpression &regex);
+
+        /**
+         * Constructs internally a regex which requires the query to start with the trigger words.
+         * Multiple words are concatenated with or, for instance: "^word1|word2|word3".
+         * The trigger words are internally escaped.
+         * Also the minLetterCount is set to the shortest word in the list.
+         * @since 5.75
+         * @see matchRegex
+         */
+        void setTriggerWords(const QStringList &triggerWords);
+
+        /**
+         * If the runner has a valid regex and non empty regex
+         * @return hasMatchRegex
+         * @since 5.75
+         */
+        bool hasMatchRegex() const;
+
     Q_SIGNALS:
         /**
          * This signal is emitted when matching is about to commence, giving runners
@@ -309,10 +429,29 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
         friend class RunnerManager;
         friend class RunnerManagerPrivate;
 
-        explicit AbstractRunner(QObject *parent = nullptr, const QString &path = QString());
-        explicit AbstractRunner(const KService::Ptr service, QObject *parent = nullptr);
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 77)
+    KRUNNER_DEPRECATED_VERSION(5, 77, "use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)")
+    explicit AbstractRunner(QObject *parent = nullptr, const QString &path = QString());
+#else
+    explicit AbstractRunner(QObject *parent = nullptr, const QString &path = QString()) = delete;
+#endif
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
+         /// @deprecated Since 5.72, use AbstractRunner(const KPluginMetaData &, QObject *)
+        KRUNNER_DEPRECATED_VERSION(5, 72, "use AbstractRunner(const KPluginMetaData &, QObject *)")
+        explicit AbstractRunner(const KService::Ptr service, QObject *parent = nullptr);
+#endif
+#endif
+        /// @since 5.72
+        explicit AbstractRunner(const KPluginMetaData &pluginMetaData, QObject *parent = nullptr);
+        /// @since 5.77
+        AbstractRunner(QObject *parent, const KPluginMetaData &pluginMetaData, const QVariantList &args);
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 77)
+         /// @deprecated Since 5.77, use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)
+        KRUNNER_DEPRECATED_VERSION(5, 77, "use AbstractRunner(QObject *, const KPluginMetaData &, const QVariantList &)")
         AbstractRunner(QObject *parent, const QVariantList &args);
+#endif
 
         /**
          * Sets whether or not the runner is available for match requests. Useful to
@@ -325,10 +464,14 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         KConfigGroup config() const;
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 71)
         /**
          * Sets whether or not the runner has options for matches
+         * @deprecated Since 5.0, this feature has been defunct
          */
+        KRUNNER_DEPRECATED_VERSION_BELATED(5, 71,  5, 0, "No longer use, feature removed")
         void setHasRunOptions(bool hasRunOptions);
+#endif
 
         /**
          * Sets the nominal speed of the runner. Only slow runners need
@@ -411,6 +554,7 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         void addSyntax(const RunnerSyntax &syntax);
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 76)
         /**
          * Set @p syntax as the default syntax for the runner; the default syntax will be
          * substituted to the empty query in single runner mode. This is also used to
@@ -424,8 +568,11 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          *
          * @param syntax the syntax to register and to set as default
          * @since 4.4
+         * @deprecated Since 5.76, feature is unused. Use addSyntax() instead.
          **/
+        KRUNNER_DEPRECATED_VERSION(5, 76, "Feature unused, use addSyntax() instead")
         void setDefaultSyntax(const RunnerSyntax &syntax);
+#endif
 
         /**
          * Sets the list of syntaxes; passing in an empty list effectively clears
@@ -436,6 +583,7 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          */
         void setSyntaxes(const QList<RunnerSyntax> &syns);
 
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 73)
         /**
          * Loads the given DataEngine
          *
@@ -454,8 +602,11 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
          *         could not be loaded
          *
          * @since 4.4
+         * @deprecated Since 5.73, DataEngines are deprecated, use e.g. a shared library to provide the data instead.
          */
+        KRUNNER_DEPRECATED_VERSION(5, 73, "DataEngines are deprecated, use e.g. a shared library to provide the data instead.")
         Q_INVOKABLE DataEngine *dataEngine(const QString &name) const;
+#endif
 
         /**
          * Reimplement this slot to run any initialization routines on first load.
@@ -477,17 +628,56 @@ class KRUNNER_EXPORT AbstractRunner : public QObject
 
 } // Plasma namespace
 
+
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 72)
+// Boilerplate to emit a version-controlled warning about the deprecated macro at leats with GCC
+#if KRUNNER_DEPRECATED_WARNINGS_SINCE >= 0x054800 // 5.72.0
+#   if defined(__GNUC__)
+#       define K_EXPORT_PLASMA_RUNNER_DO_PRAGMA(x) _Pragma (#x)
+#       define K_EXPORT_PLASMA_RUNNER_WARNING(x) K_EXPORT_PLASMA_RUNNER_DO_PRAGMA(message(#x))
+#   else
+#       define K_EXPORT_PLASMA_RUNNER_WARNING(x)
+#   endif
+#else
+#   define K_EXPORT_PLASMA_RUNNER_WARNING(x)
+#endif
+/**
+ * @relates Plasma::AbstractRunner
+ *
+ * Registers a runner plugin.
+ *
+ * @deprecated Since 5.72, use K_EXPORT_PLASMA_RUNNER_WITH_JSON(classname, jsonFile) instead
+ */
 #define K_EXPORT_PLASMA_RUNNER( libname, classname )     \
+K_EXPORT_PLASMA_RUNNER_WARNING("Deprecated. Since 5.72, use K_EXPORT_PLASMA_RUNNER_WITH_JSON(classname, jsonFile) instead") \
 K_PLUGIN_FACTORY(factory, registerPlugin<classname>();) \
 K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+#endif
 
+/**
+ * @relates Plasma::AbstractRunner
+ *
+ * Registers a runner plugin with JSON metadata.
+ *
+ * @param classname name of the AbstractRunner derived class.
+ * @param jsonFile name of the JSON file to be compiled into the plugin as metadata
+ *
+ * @since 5.72
+ */
+#define K_EXPORT_PLASMA_RUNNER_WITH_JSON(classname, jsonFile) \
+    K_PLUGIN_FACTORY_WITH_JSON(classname ## Factory, jsonFile, registerPlugin<classname>();) \
+    K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+
+#if KRUNNER_ENABLE_DEPRECATED_SINCE(5, 75)
 /**
  * These plugins are Used by the plugin selector dialog to show
  * configuration options specific to this runner. These options
  * must be runner global and not pertain to a specific match.
+ * @deprecated Since 5.0, use K_PLUGIN_FACTORY directly
  */
 #define K_EXPORT_RUNNER_CONFIG( name, classname )     \
 K_PLUGIN_FACTORY(ConfigFactory, registerPlugin<classname>();) \
 K_EXPORT_PLUGIN_VERSION(PLASMA_VERSION)
+#endif
 
 #endif
