@@ -1,49 +1,31 @@
 /*
- *   Copyright 2006-2007 Aaron Seigo <aseigo@kde.org>
- *
- *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU Library General Public License as
- *   published by the Free Software Foundation; either version 2, or
- *   (at your option) any later version.
- *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details
- *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this program; if not, write to the
- *   Free Software Foundation, Inc.,
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2006-2007 Aaron Seigo <aseigo@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-or-later
+*/
 
 #include "runnercontext.h"
 
 #include <cmath>
 
 #include <QReadWriteLock>
-
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QRegularExpression>
 #include <QSharedData>
+#include <QMimeDatabase>
+#include <QStandardPaths>
+#include <QUrl>
+
+#include <KConfigGroup>
+#include <KShell>
+#include <KProtocolInfo>
 
-#include <kconfiggroup.h>
 #include "krunner_debug.h"
-#include <qmimedatabase.h>
-#include <kshell.h>
-#include <qstandardpaths.h>
-#include <qurl.h>
-#include <kprotocolinfo.h>
-
 #include "abstractrunner.h"
 #include "querymatch.h"
 
-//#define LOCK_FOR_READ(d) if (d->policy == Shared) { d->lock.lockForRead(); }
-//#define LOCK_FOR_WRITE(d) if (d->policy == Shared) { d->lock.lockForWrite(); }
-//#define UNLOCK(d) if (d->policy == Shared) { d->lock.unlock(); }
-
 #define LOCK_FOR_READ(d) d->lock.lockForRead();
 #define LOCK_FOR_WRITE(d) d->lock.lockForWrite();
 #define UNLOCK(d) d->lock.unlock();
@@ -51,109 +33,6 @@
 namespace Plasma
 {
 
-/*
-Corrects the case of the last component in a path (e.g. /usr/liB -> /usr/lib)
-path: The path to be processed.
-correctCasePath: The corrected-case path
-mustBeDir: Tells whether the last component is a folder or doesn't matter
-Returns true on success and false on error, in case of error, correctCasePath is not modified
-*/
-bool correctLastComponentCase(const QString &path, QString &correctCasePath, const bool mustBeDir)
-{
-    //qCDebug(KRUNNER) << "Correcting " << path;
-
-    // If the file already exists then no need to search for it.
-    if (QFile::exists(path)) {
-        correctCasePath = path;
-        //qCDebug(KRUNNER) << "Correct path is" << correctCasePath;
-        return true;
-    }
-
-    const QFileInfo pathInfo(path);
-
-    const QDir fileDir = pathInfo.dir();
-    //qCDebug(KRUNNER) << "Directory is" << fileDir;
-
-    const QString filename = pathInfo.fileName();
-    //qCDebug(KRUNNER) << "Filename is" << filename;
-
-    //qCDebug(KRUNNER) << "searching for a" << (mustBeDir ? "directory" : "directory/file");
-
-    const QStringList matchingFilenames = fileDir.entryList(QStringList(filename),
-                                          mustBeDir ? QDir::Dirs : QDir::NoFilter);
-
-    if (matchingFilenames.empty()) {
-        //qCDebug(KRUNNER) << "No matches found!!\n";
-        return false;
-    } else {
-        /*if (matchingFilenames.size() > 1) {
-#ifndef NDEBUG
-            // qCDebug(KRUNNER) << "Found multiple matches!!\n";
-#endif
-        }*/
-
-        if (fileDir.path().endsWith(QDir::separator())) {
-            correctCasePath = fileDir.path() + matchingFilenames[0];
-        } else {
-            correctCasePath = fileDir.path() + QDir::separator() + matchingFilenames[0];
-        }
-
-        //qCDebug(KRUNNER) << "Correct path is" << correctCasePath;
-        return true;
-    }
-}
-
-/*
-Corrects the case of a path (e.g. /uSr/loCAL/bIN -> /usr/local/bin)
-path: The path to be processed.
-corrected: The corrected-case path
-Returns true on success and false on error, in case of error, corrected is not modified
-*/
-bool correctPathCase(const QString& path, QString &corrected)
-{
-    // early exit check
-    if (QFile::exists(path)) {
-        corrected = path;
-        return true;
-    }
-
-    // path components
-    QStringList components = QString(path).split(QDir::separator());
-
-    if (components.size() < 1) {
-        return false;
-    }
-
-    const bool mustBeDir = components.back().isEmpty();
-
-    //qCDebug(KRUNNER) << "Components are" << components;
-
-    if (mustBeDir) {
-        components.pop_back();
-    }
-
-    if (components.isEmpty()) {
-        return true;
-    }
-
-    QString correctPath;
-    const int initialComponents = components.size();
-    for (int i = 0; i < initialComponents - 1; i++) {
-        const QString tmp = components[0] + QDir::separator() + components[1];
-
-        if (!correctLastComponentCase(tmp, correctPath, components.size() > 2 || mustBeDir)) {
-            //qCDebug(KRUNNER) << "search was not successful";
-            return false;
-        }
-
-        components.removeFirst();
-        components[0] = correctPath;
-    }
-
-    corrected = correctPath;
-    return true;
-}
-
 class RunnerContextPrivate : public QSharedData
 {
     public:
@@ -172,13 +51,13 @@ class RunnerContextPrivate : public QSharedData
               q(p.q),
               singleRunnerQueryMode(false)
         {
-            //qCDebug(KRUNNER) << "¿¿¿¿¿¿¿¿¿¿¿¿¿¿¿¿¿boo yeah" << type;
         }
 
         ~RunnerContextPrivate()
         {
         }
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
         /**
          * Determines type of query
                     &&
@@ -199,6 +78,7 @@ class RunnerContextPrivate : public QSharedData
                                      RunnerContext::Executable;
             } else {
                 QUrl url = QUrl::fromUserInput(term);
+
                 // QUrl::fromUserInput assigns http to everything if it cannot match it to
                 // anything else. We do not want that.
                 if (url.scheme() == QLatin1String("http")) {
@@ -206,8 +86,9 @@ class RunnerContextPrivate : public QSharedData
                         url.setScheme(QString());
                     }
                 }
+
                 const bool hasProtocol = !url.scheme().isEmpty();
-                const bool isLocalProtocol = hasProtocol && KProtocolInfo::protocolClass(url.scheme()) == QLatin1String(":local");
+                const bool isLocalProtocol = !hasProtocol || KProtocolInfo::protocolClass(url.scheme()) == QLatin1String(":local");
                 if ((hasProtocol &&
                     ((!isLocalProtocol && !url.host().isEmpty()) ||
                      (isLocalProtocol && url.scheme() != QLatin1String("file"))))
@@ -220,47 +101,51 @@ class RunnerContextPrivate : public QSharedData
                     // at this point in the game, we assume we have a path,
                     // but if a path doesn't have any slashes
                     // it's too ambiguous to be sure we're in a filesystem context
-                    path = QDir::cleanPath(url.toLocalFile());
-                    //qCDebug(KRUNNER)<< "slash check" << path;
-                    if (hasProtocol || ((path.indexOf(QLatin1Char('/')) != -1 || path.indexOf(QLatin1Char('\\')) != -1))) {
-                        QString correctCasePath;
-                        if (correctPathCase(path, correctCasePath)) {
-                            path = correctCasePath;
-                            QFileInfo info(path);
-                            //qCDebug(KRUNNER)<< "correct cas epath is" << correctCasePath << info.isSymLink() <<
-                            //    info.isDir() << info.isFile();
-
-                            if (info.isSymLink()) {
-                                path = info.canonicalFilePath();
-                                info = QFileInfo(path);
-                            }
-                            if (info.isDir()) {
-                                type = RunnerContext::Directory;
-                                mimeType = QStringLiteral("inode/folder");
-                            } else if (info.isFile()) {
-                                type = RunnerContext::File;
-                                QMimeDatabase db;
-                                QMimeType mime = db.mimeTypeForFile(path);
-                                if (!mime.isDefault()) {
-                                    mimeType = mime.name();
-                                }
+                    path = !url.scheme().isEmpty() ? QDir::cleanPath(url.toLocalFile()) : path;
+                    if ((path.indexOf(QLatin1Char('/')) != -1 || path.indexOf(QLatin1Char('\\')) != -1)) {
+                        QFileInfo info(path);
+                        if (info.isSymLink()) {
+                            path = info.canonicalFilePath();
+                            info = QFileInfo(path);
+                        }
+                        if (info.isDir()) {
+                            type = RunnerContext::Directory;
+                            mimeType = QStringLiteral("inode/folder");
+                        } else if (info.isFile()) {
+                            type = RunnerContext::File;
+                            QMimeDatabase db;
+                            QMimeType mime = db.mimeTypeForFile(path);
+                            if (!mime.isDefault()) {
+                                mimeType = mime.name();
                             }
                         }
                     }
                 }
             }
-
-            //qCDebug(KRUNNER) << "term2type" << term << type;
         }
+#endif
 
         void invalidate()
         {
             q = &s_dummyContext;
         }
 
+        QueryMatch* findMatchById(const QString &id)
+        {
+            QueryMatch *match = nullptr;
+            lock.lockForRead();
+            for (auto &matchEntry : matches) {
+                if (matchEntry.id() == id) {
+                    match = &matchEntry;
+                    break;
+                }
+            }
+            lock.unlock();
+            return match;
+        }
+
         QReadWriteLock lock;
         QList<QueryMatch> matches;
-        QMap<QString, const QueryMatch*> matchesById;
         QHash<QString, int> launchCounts;
         QString term;
         QString mimeType;
@@ -329,16 +214,14 @@ void RunnerContext::reset()
     // ref count was 1 (e.g. only the RunnerContext is using
     // the dptr) then we won't get a copy made
     if (!d->matches.isEmpty()) {
-        d->matchesById.clear();
         d->matches.clear();
-        emit matchesChanged();
+        Q_EMIT matchesChanged();
     }
 
     d->term.clear();
     d->mimeType.clear();
     d->type = UnknownType;
     d->singleRunnerQueryMode = false;
-    //qCDebug(KRUNNER) << "match count" << d->matches.count();
 }
 
 void RunnerContext::setQuery(const QString &term)
@@ -350,7 +233,9 @@ void RunnerContext::setQuery(const QString &term)
     }
 
     d->term = term;
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
     d->determineType();
+#endif
 }
 
 QString RunnerContext::query() const
@@ -360,26 +245,34 @@ QString RunnerContext::query() const
     // and setQuery(QString) calls reset()
     return d->term;
 }
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 
 void RunnerContext::setEnabledCategories(const QStringList& categories)
 {
     d->enabledCategories = categories;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QStringList RunnerContext::enabledCategories() const
 {
     return d->enabledCategories;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 RunnerContext::Type RunnerContext::type() const
 {
     return d->type;
 }
+#endif
 
+#if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QString RunnerContext::mimeType() const
 {
     return d->mimeType;
 }
+#endif
 
 bool RunnerContext::isValid() const
 {
@@ -406,19 +299,12 @@ bool RunnerContext::addMatches(const QList<QueryMatch> &matches)
         }
 
         d->matches.append(match);
-#ifndef NDEBUG
-        if (d->matchesById.contains(match.id())) {
-            // qCDebug(KRUNNER) << "Duplicate match id " << match.id() << "from" << match.runner()->name();
-        }
-#endif
-        d->matchesById.insert(match.id(), &d->matches.at(d->matches.size() - 1));
     }
     UNLOCK(d);
-    //qCDebug(KRUNNER)<< "add matches";
     // A copied searchContext may share the d pointer,
     // we always want to sent the signal of the object that created
     // the d pointer
-    emit d->q->matchesChanged();
+    Q_EMIT d->q->matchesChanged();
 
     return true;
 }
@@ -439,49 +325,22 @@ bool RunnerContext::addMatch(const QueryMatch &match)
     }
 
     d->matches.append(m);
-    d->matchesById.insert(m.id(), &d->matches.at(d->matches.size() - 1));
     UNLOCK(d);
-    //qCDebug(KRUNNER)<< "added match" << match->text();
-    emit d->q->matchesChanged();
+    Q_EMIT d->q->matchesChanged();
 
     return true;
 }
 
 bool RunnerContext::removeMatches(const QStringList matchIdList)
 {
-    if (!isValid()) {
-        return false;
-    }
-
-    QStringList presentMatchIdList;
-    QList<const QueryMatch*> presentMatchList;
-
-    LOCK_FOR_READ(d)
-    for (const QString &matchId : matchIdList) {
-        const QueryMatch* match = d->matchesById.value(matchId, nullptr);
-        if (match) {
-            presentMatchList << match;
-            presentMatchIdList << matchId;
+    bool success = false;
+    for (const QString &id : matchIdList) {
+        if (removeMatch(id)) {
+            success = true;
         }
     }
-    UNLOCK(d)
 
-    if (presentMatchIdList.isEmpty()) {
-        return false;
-    }
-
-    LOCK_FOR_WRITE(d)
-    for (const QueryMatch *match : qAsConst(presentMatchList)) {
-        d->matches.removeAll(*match);
-    }
-    for (const QString &matchId : qAsConst(presentMatchIdList)) {
-        d->matchesById.remove(matchId);
-    }
-    UNLOCK(d)
-
-    emit d->q->matchesChanged();
-
-    return true;
+    return success;
 }
 
 bool RunnerContext::removeMatch(const QString matchId)
@@ -490,16 +349,15 @@ bool RunnerContext::removeMatch(const QString matchId)
         return false;
     }
     LOCK_FOR_READ(d)
-    const QueryMatch* match = d->matchesById.value(matchId, nullptr);
+    const QueryMatch* match = d->findMatchById(matchId);
     UNLOCK(d)
     if (!match) {
         return false;
     }
     LOCK_FOR_WRITE(d)
     d->matches.removeAll(*match);
-    d->matchesById.remove(matchId);
     UNLOCK(d)
-    emit d->q->matchesChanged();
+    Q_EMIT d->q->matchesChanged();
 
     return true;
 }
@@ -526,12 +384,11 @@ bool RunnerContext::removeMatches(Plasma::AbstractRunner *runner)
 
     LOCK_FOR_WRITE(d)
     for (const QueryMatch &match : qAsConst(presentMatchList)) {
-        d->matchesById.remove(match.id());
         d->matches.removeAll(match);
     }
     UNLOCK(d)
 
-    emit d->q->matchesChanged();
+    Q_EMIT d->q->matchesChanged();
     return true;
 }
 
@@ -546,14 +403,9 @@ QList<QueryMatch> RunnerContext::matches() const
 QueryMatch RunnerContext::match(const QString &id) const
 {
     LOCK_FOR_READ(d)
-    const QueryMatch *match = d->matchesById.value(id, nullptr);
+    const QueryMatch *match = d->findMatchById(id);
     UNLOCK(d)
-
-    if (match) {
-        return *match;
-    }
-
-    return QueryMatch(nullptr);
+    return match ? *match : QueryMatch(nullptr);
 }
 
 void RunnerContext::setSingleRunnerQueryMode(bool enabled)
