@@ -20,6 +20,12 @@
 #include <KPluginMetaData>
 #include <KConfigWatcher>
 
+
+#include "config.h"
+#ifdef HAVE_KACTIVITIES
+#include <KActivities/Consumer>
+#endif
+
 #include <ThreadWeaver/DebuggingAids>
 #include <ThreadWeaver/Queue>
 #include <ThreadWeaver/Thread>
@@ -118,21 +124,31 @@ public:
 
     void loadConfiguration()
     {
-        KConfigGroup config = configGroup();
-
         // Limit the number of instances of a single normal speed runner and all of the slow runners
         // to half the number of threads
         DefaultRunnerPolicy::instance().setCap(qMax(2, Queue::instance()->maximumNumberOfThreads() / 2));
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
-        enabledCategories = config.readEntry("enabledCategories", QStringList());
+        enabledCategories = stateData.readEntry("enabledCategories", QStringList());
 #endif
-        context.restore(config);
-    }
-
-    KConfigGroup configGroup()
-    {
-        return conf.isValid() ? conf : KConfigGroup(KSharedConfig::openConfig(configFile), "PlasmaRunnerManager");
+#ifdef HAVE_KACTIVITIES
+        // Wait for consumer to be ready
+        QObject::connect(&activitiesConsumer, &KActivities::Consumer::serviceStatusChanged, &activitiesConsumer,
+            [this](KActivities::Consumer::ServiceStatus status) {
+            if (status == KActivities::Consumer::Running) {
+                deleteHistoryOfDeletedActivities();
+            }
+        });
+#endif
+        const KConfigGroup generalConfig = configPrt->group("General");
+        const bool _historyEnabled = generalConfig.readEntry("HistoryEnabled", true);
+        if (historyEnabled != _historyEnabled) {
+            historyEnabled = _historyEnabled;
+            Q_EMIT q->historyEnabledChanged();
+        }
+        activityAware = generalConfig.readEntry("ActivityAware", true);
+        retainPriorSearch = generalConfig.readEntry("RetainPriorSearch", true);
+        context.restore(stateData);
     }
 
     void clearSingleRunner()
@@ -207,17 +223,11 @@ QT_WARNING_POP
 
     void loadRunners()
     {
-        KConfigGroup config = configGroup();
         QVector<KPluginMetaData> offers = RunnerManager::runnerMetaDataList();
 
-        const bool loadAll = config.readEntry("loadAll", false);
+        const bool loadAll = stateData.readEntry("loadAll", false);
         const bool noWhiteList = whiteList.isEmpty();
-        KConfigGroup pluginConf;
-        if (conf.isValid()) {
-            pluginConf = KConfigGroup(&conf, "Plugins");
-        } else {
-            pluginConf = KConfigGroup(KSharedConfig::openConfig(configFile), "Plugins");
-        }
+        KConfigGroup pluginConf = configPrt->group("Plugins");
 
         advertiseSingleRunnerIds.clear();
 
@@ -476,6 +486,68 @@ QT_WARNING_POP
             searchJobs.insert(job);
     }
 
+    inline QString getActivityKey()
+    {
+#ifdef HAVE_KACTIVITIES
+        return activityAware ? activitiesConsumer.currentActivity() : nulluuid;
+#else
+        return nulluuid;
+#endif
+    }
+
+    void addToHistory()
+    {
+        const QString term = context.query();
+        // We want to imitate the shall behavior
+        if (!historyEnabled || term.isEmpty() || untrimmedTerm.startsWith(QLatin1Char(' '))) {
+            return;
+        }
+        QStringList historyEntries = readHistoryForCurrentActivity();
+        // Avoid removing the same item from the front and prepending it again
+        if (!historyEntries.isEmpty() && historyEntries.constFirst() == term) {
+            return;
+        }
+
+        historyEntries.removeOne(term);
+        historyEntries.prepend(term);
+
+        while (historyEntries.count() > 50) { // we don't want to store more than 50 entries
+            historyEntries.removeLast();
+        }
+        writeActivityHistory(historyEntries);
+    }
+
+    void writeActivityHistory(const QStringList &historyEntries)
+    {
+        stateData.group("History").writeEntry(getActivityKey(), historyEntries, KConfig::Notify);
+        stateData.sync();
+    }
+
+#ifdef HAVE_KACTIVITIES
+    void deleteHistoryOfDeletedActivities()
+    {
+        KConfigGroup historyGroup = stateData.group("History");
+        QStringList historyEntries = historyGroup.keyList();
+        historyEntries.removeOne(nulluuid);
+
+        // Check if history still exists
+        const QStringList activities = activitiesConsumer.activities();
+        for (const auto &a : activities) {
+            historyEntries.removeOne(a);
+        }
+
+        for (const QString &deletedActivity : qAsConst(historyEntries)) {
+            historyGroup.deleteEntry(deletedActivity);
+        }
+        historyGroup.sync();
+    }
+#endif
+
+    inline QStringList readHistoryForCurrentActivity()
+    {
+        return stateData.group("History").readEntry(getActivityKey(), QStringList());
+    }
+
     // Delay in ms before slow runners are allowed to run
     static const int slowRunDelay = 400;
 
@@ -490,7 +562,6 @@ QT_WARNING_POP
     AbstractRunner* currentSingleRunner;
     QSet<QSharedPointer<FindMatchesJob> > searchJobs;
     QSet<QSharedPointer<FindMatchesJob> > oldSearchJobs;
-    KConfigGroup conf;
     QStringList enabledCategories;
     QString singleModeRunnerId;
     bool prepped : 1;
@@ -499,9 +570,20 @@ QT_WARNING_POP
     bool teardownRequested : 1;
     bool singleMode : 1;
     bool singleRunnerWasLoaded : 1;
+    bool activityAware :1;
+    bool historyEnabled :1;
+    bool retainPriorSearch :1;
     QStringList whiteList;
     QString configFile;
     KConfigWatcher::Ptr watcher;
+    QHash<QString, QString> priorSearch;
+    QString untrimmedTerm;
+    QString nulluuid = QStringLiteral("00000000-0000-0000-0000-000000000000");
+    KSharedConfigPtr configPrt;
+    KConfigGroup stateData;
+#ifdef HAVE_KACTIVITIES
+    const KActivities::Consumer activitiesConsumer;
+#endif
 };
 
 /*****************************************************
@@ -517,7 +599,16 @@ RunnerManager::RunnerManager(const QString &configFile, QObject *parent)
     : QObject(parent),
       d(new RunnerManagerPrivate(this))
 {
-    d->configFile = configFile;
+    d->configPrt = KSharedConfig::openConfig(configFile);
+    // If the old config group still exists the migration script wasn't executed
+    // so we keep using this location
+    KConfigGroup oldStateDataGroup = d->configPrt->group("PlasmaRunnerManager");
+    if (oldStateDataGroup.exists() && !oldStateDataGroup.readEntry("migrated", false)) {
+        d->stateData = oldStateDataGroup;
+    } else {
+        d->stateData = KSharedConfig::openConfig(QStringLiteral("krunnerstaterc"), KConfig::NoGlobals,
+                                                 QStandardPaths::GenericDataLocation)->group("PlasmaRunnerManager");
+    }
     d->loadConfiguration();
 }
 
@@ -526,7 +617,8 @@ RunnerManager::RunnerManager(KConfigGroup &c, QObject *parent)
     : QObject(parent),
       d(new RunnerManagerPrivate(this))
 {
-    d->conf = KConfigGroup(&c, "PlasmaRunnerManager");
+    d->configPrt = KSharedConfig::openConfig();
+    d->stateData = KConfigGroup(&c, "PlasmaRunnerManager");
     d->loadConfiguration();
 }
 #endif
@@ -542,7 +634,8 @@ RunnerManager::~RunnerManager()
 
 void RunnerManager::reloadConfiguration()
 {
-    KSharedConfig::openConfig()->reparseConfiguration();
+    d->configPrt->reparseConfiguration();
+    d->stateData.config()->reparseConfiguration();
     d->loadConfiguration();
     d->loadRunners();
 }
@@ -559,9 +652,7 @@ void RunnerManager::setAllowedRunners(const QStringList &runners)
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 void RunnerManager::setEnabledCategories(const QStringList& categories)
 {
-    KConfigGroup config = d->configGroup();
-    config.writeEntry("enabledCategories", categories);
-
+    d->stateData.writeEntry("enabledCategories", categories);
     d->enabledCategories = categories;
 
     if (!d->runners.isEmpty()) {
@@ -572,15 +663,13 @@ void RunnerManager::setEnabledCategories(const QStringList& categories)
 
 QStringList RunnerManager::allowedRunners() const
 {
-    KConfigGroup config = d->configGroup();
-    return config.readEntry("pluginWhiteList", QStringList());
+    return d->stateData.readEntry("pluginWhiteList", QStringList());
 }
 
 #if KRUNNER_BUILD_DEPRECATED_SINCE(5, 76)
 QStringList RunnerManager::enabledCategories() const
 {
-    KConfigGroup config = d->configGroup();
-    QStringList list = config.readEntry("enabledCategories", QStringList());
+    QStringList list = d->stateData.readEntry("enabledCategories", QStringList());
     if (list.isEmpty()) {
         list.reserve(d->runners.count());
         for (AbstractRunner* runner : qAsConst(d->runners)) {
@@ -736,6 +825,20 @@ void RunnerManager::run(const QueryMatch &match)
     d->context.run(match);
 }
 
+bool RunnerManager::runMatch(const QueryMatch &match)
+{
+    d->addToHistory();
+    if (match.type() == Plasma::QueryMatch::InformationalMatch) {
+        const QString info = match.data().toString();
+        if (!info.isEmpty()) {
+            Q_EMIT setSearchTerm(info, info.length());
+            return false;
+        }
+    }
+    d->context.run(match);
+    return true;
+}
+
 QList<QAction*> RunnerManager::actionsForMatch(const QueryMatch &match)
 {
     if (AbstractRunner *runner = match.runner()) {
@@ -841,8 +944,7 @@ void RunnerManager::matchSessionComplete()
     d->checkTearDown();
     // We save the context config after each session, just like the history entries
     // BUG: 424505
-    KConfigGroup config = d->configGroup();
-    d->context.save(config);
+    d->context.save(d->stateData);
 }
 
 void RunnerManager::launchQuery(const QString &term)
@@ -854,6 +956,7 @@ void RunnerManager::launchQuery(const QString &untrimmedTerm, const QString &run
 {
     setupMatchSession();
     QString term = untrimmedTerm.trimmed();
+    d->untrimmedTerm = untrimmedTerm;
 
     setSingleModeRunnerId(runnerName);
     setSingleMode(d->currentSingleRunner);
@@ -922,6 +1025,31 @@ QString RunnerManager::query() const
     return d->context.query();
 }
 
+QStringList RunnerManager::history() const
+{
+    return d->readHistoryForCurrentActivity();
+}
+
+void RunnerManager::removeFromHistory(int index)
+{
+    QStringList changedHistory = history();
+    if (index < changedHistory.length()) {
+        changedHistory.removeAt(index);
+        d->writeActivityHistory(changedHistory);
+    }
+}
+
+QString RunnerManager::getHistorySuggestion(const QString &typedQuery) const
+{
+    const QStringList historyList = history();
+    for (const QString &entry : historyList) {
+        if (entry.startsWith(typedQuery, Qt::CaseInsensitive)) {
+            return entry;
+        }
+    }
+    return QString();
+}
+
 void RunnerManager::reset()
 {
     // If ThreadWeaver is idle, it is safe to clear previous jobs
@@ -949,7 +1077,7 @@ void RunnerManager::reset()
 void RunnerManager::enableKNotifyPluginWatcher()
 {
     if (!d->watcher) {
-        d->watcher = KConfigWatcher::create(KSharedConfig::openConfig(d->configGroup().config()->name()));
+        d->watcher = KConfigWatcher::create(d->configPrt);
         connect(d->watcher.data(), &KConfigWatcher::configChanged, this, [this](const KConfigGroup &group, const QByteArrayList &changedNames) {
             const QString groupName = group.name();
             if (groupName == QLatin1String("Plugins")) {
@@ -973,6 +1101,26 @@ void RunnerManager::enableKNotifyPluginWatcher()
     }
 }
 
+QString RunnerManager::priorSearch() const
+{
+    return d->priorSearch.value(d->getActivityKey());
+}
+
+void RunnerManager::setPriorSearch(const QString &search)
+{
+    d->priorSearch.insert(d->getActivityKey(), search);
+}
+
+bool RunnerManager::historyEnabled()
+{
+    return d->historyEnabled;
+}
+
+bool RunnerManager::retainPriorSearch()
+{
+    return d->retainPriorSearch;
+}
+
 } // Plasma namespace
 
 
